/*
Author: Mark Amo-Boateng
Contact: m.amoboateng@gmail.com
Create Date: 6th June 2013
Description: 
A fast and efficient method to create, 
allocate and resize 1-D and 2-D dynamic arrays in C/C++. 
This simplifies the work of manually creating dynamic arrays 
where the use of standard methods may not be applicable of efficient. 

License : GNU

*/

#pragma once 
# include "Header.h"


template <typename T> 
T *AllocateDynamic1DArray( int size, T value = 0, bool set_values = true, bool fill_index = false)
{
      T *dynamicArray;
      dynamicArray = new T[size+1];
	  if (set_values) for(int i = 0; i < size; i++) dynamicArray[i] = value;  
	  if (fill_index) for(int i = 0; i < size; i++) dynamicArray[i] = value + i; 
      return dynamicArray;
}

template <typename T>
void FreeDynamic1DArray(T* dArray)
{
	delete [] dArray;   
	//dArray = NULL;
}

template <typename T>
void safeFreeDynamic1DArray(T* dArray)
{
     if (dArray != NULL) delete [] dArray;   
	 dArray = NULL;
}

template <typename T> 
T *ReAllocDynamic1DArray( int new_size, T *old_dynamicArray, T init_value, bool set_default_values = true)
{
      T *new_dynamicArray = AllocateDynamic1DArray<T>(new_size,init_value,set_default_values);
	  FreeDynamic1DArray<T>(old_dynamicArray);	  
	  old_dynamicArray = new_dynamicArray;     
      return new_dynamicArray;
}

template <typename T> 
T *ResizeDynamic1DArray_Adv( int new_size, T *old_dynamicArray, T init_value, bool set_default_values = true, bool copy_old_contents = false, int copy_size =0, int copy_start = 0)
{
      T *new_dynamicArray;
      new_dynamicArray = new T[new_size];
	  if (set_default_values) for(int i = 0; i < new_size; i++) new_dynamicArray[i] = init_value;
	  if (copy_old_contents) for(int i = 0; i < copy_size; i++) new_dynamicArray[i] = old_dynamicArray[copy_start + i];

	  //FreeDynamic1DArray<T>(old_dynamicArray);
	  delete [] old_dynamicArray;   old_dynamicArray = NULL;
	  old_dynamicArray = new_dynamicArray;     
      return new_dynamicArray;
}

template <typename T> 
T **AllocateDynamic2DArray( int nRows, int nCols, T init_value = 0, bool set_values = true)
{
      T **dynamicArray;
      dynamicArray = new T*[nRows];
      for( int i = 0 ; i < nRows ; i++ ) 
	  {
		 dynamicArray[i] = new T [nCols];
		 if (set_values) for(int j = 0; j < nCols; j++) dynamicArray[i][j] = init_value;
	  }
      return dynamicArray;
}

template <typename T>
void FreeDynamic2DArray(T** dArray, int nRows)
{
	for( int i = 0 ; i < nRows ; i++ )
	{
		delete [] dArray[i]; 
	}
    delete [] dArray;   
}

template <typename T>
void safeFreeDynamic2DArray(T** dArray, int nRows)
{
	for( int i = 0 ; i < nRows ; i++ )
	{
		if (dArray[i] != NULL) delete [] dArray[i];
		//delete [] dArray[i]; 
		dArray[i] = NULL;
	}
    delete [] dArray;   
	dArray = NULL;
}

template <typename T>
void NullDynamic2DArray(T** dArray, int nRows)
{
	for( int i = 0 ; i < nRows ; i++ )
	{		
		dArray[i] = NULL;
	}     
	dArray = NULL;
}
